package ld27.render;

import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL15.*;

import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.ArrayList;

import org.lwjgl.BufferUtils;
import org.lwjgl.opengl.Display;
import org.lwjgl.util.glu.GLU;

public class Renderer {
	private int vertexBuffer, uvBuffer, colorBuffer, normalBuffer;
	public ArrayList<Float> vertices, uvs, colors, normals;
	private int numberOfVertices, numberOfBufferVertices;
	private boolean useUVs, useColors, useNormals;
	public float r, g, b, a;
	private int drawMode;
	
	public Renderer(boolean useUVs, boolean useColors, boolean useNormals) {
		resetArrays(useUVs, useColors, useNormals);
	}
	
	public void setColor(float r, float g, float b, float a) {
		this.r = r;
		this.g = g;
		this.b = b;
		this.a = a;
	}
	
	public void addVertex(float[] vertex, float[] uv, float[] normal) {
		for(int i = 0; i < 3; i++) {
			if(vertex != null && vertex.length > i) {
				vertices.add(vertex[i]);
			}else{
				vertices.add(0f);
			}
		}
		
		if(useUVs) {
			for(int i = 0; i < 2; i++) {
				if(uv != null && uv.length > i) {
					uvs.add(uv[i]);
				}else{
					uvs.add(0f);
				}
			}
		}
		
		if(useColors) {
			colors.add(r);
			colors.add(g);
			colors.add(b);
			colors.add(a);
		}
		
		if(useNormals) {
			for(int i = 0; i < 3; i++) {
				if(normal != null && normal.length > i) {
					normals.add(normal[i]);
				}else{
					normals.add(0f);
				}
			}
		}
		numberOfVertices++;
	}
	
	private FloatBuffer toBuffer(ArrayList<Float> in) {
		FloatBuffer buffer = BufferUtils.createFloatBuffer(in.size());
		for(Float f : in) {
			buffer.put(f);
		}
		buffer.flip();
		return buffer;
	}
	
	private IntBuffer toBuffer(int[] in) {
		IntBuffer buffer = BufferUtils.createIntBuffer(in.length);
		for(int f : in) {
			buffer.put(f);
		}
		buffer.flip();
		return buffer;
	}
	
	public void genBuffers(int drawMode) {
		resetBuffers();
		this.drawMode = drawMode;
		
		vertexBuffer = glGenBuffers();
		glBindBuffer(GL_ARRAY_BUFFER, vertexBuffer);
		glBufferData(GL_ARRAY_BUFFER, toBuffer(vertices), GL_STATIC_DRAW);
		
		if(useUVs) {
			uvBuffer = glGenBuffers();
			glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
			glBufferData(GL_ARRAY_BUFFER, toBuffer(uvs), GL_STATIC_DRAW);
		}

		if(useColors) {
			colorBuffer = glGenBuffers();
			glBindBuffer(GL_ARRAY_BUFFER, colorBuffer);
			glBufferData(GL_ARRAY_BUFFER, toBuffer(colors), GL_STATIC_DRAW);
		}

		if(useNormals) {
			normalBuffer = glGenBuffers();
			glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
			glBufferData(GL_ARRAY_BUFFER, toBuffer(normals), GL_STATIC_DRAW);
		}
		
		glBindBuffer(GL_ARRAY_BUFFER, 0);
		numberOfBufferVertices = numberOfVertices;
	}
	
	public void drawBuffers() {
		drawBuffers(null);
	}
	
	public void drawBuffers(int[] indices) {
		if(vertexBuffer == 0 || 
				(useUVs && uvBuffer == 0) || 
				(useColors && colorBuffer == 0) || 
				(useNormals && normalBuffer == 0)) {
			return;
		}
		
		if(!useColors) { glColor4f(r, g, b, a); }
		
		glEnableClientState(GL_VERTEX_ARRAY);
		if(useUVs) { glEnableClientState(GL_TEXTURE_COORD_ARRAY); }
		if(useColors) { glEnableClientState(GL_COLOR_ARRAY); }
		if(useNormals) { glEnableClientState(GL_NORMAL_ARRAY); }
		
		glBindBuffer(GL_ARRAY_BUFFER, vertexBuffer);
		glVertexPointer(3, GL_FLOAT, 0, 0);
		
		if(useUVs) {
			glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
			glTexCoordPointer(2, GL_FLOAT, 0, 0);
		}
		
		if(useColors) {
			glBindBuffer(GL_ARRAY_BUFFER, colorBuffer);
			glColorPointer(4, GL_FLOAT, 0, 0);
		}
		
		if(useNormals) {
			glBindBuffer(GL_ARRAY_BUFFER, normalBuffer);
			glColorPointer(3, GL_FLOAT, 0, 0);
		}
		
		if(indices == null) {
			glDrawArrays(drawMode, 0, numberOfBufferVertices);
		}else{
			glDrawElements(drawMode, toBuffer(indices));
		}
		
		glDisableClientState(GL_VERTEX_ARRAY);
		if(useUVs) { glDisableClientState(GL_TEXTURE_COORD_ARRAY); }
		if(useColors) { glDisableClientState(GL_COLOR_ARRAY); }
		if(useNormals) { glDisableClientState(GL_NORMAL_ARRAY); }
	}
	
	public void resetArrays() {
		vertices = new ArrayList<Float>();
		if(useUVs) { uvs = new ArrayList<Float>(); }else{ uvs = null; }
		if(useColors) { colors = new ArrayList<Float>(); }else{ colors = null; }
		if(useNormals) { normals = new ArrayList<Float>(); }else{ normals = null; }
		numberOfVertices = 0;
	}
	
	public void resetArrays(boolean useUVs, boolean useColors, boolean useNormals) {
		this.useUVs = useUVs;
		this.useColors = useColors;
		this.useNormals = useNormals;
		resetArrays();
	}
	
	public void resetBuffers() {
		if(vertexBuffer != 0) { glDeleteBuffers(vertexBuffer); vertexBuffer = 0; }
		if(uvBuffer != 0) { glDeleteBuffers(uvBuffer); uvBuffer = 0; }
		if(colorBuffer != 0) { glDeleteBuffers(colorBuffer); colorBuffer = 0; }
		if(normalBuffer != 0) { glDeleteBuffers(normalBuffer); normalBuffer = 0; }
		numberOfBufferVertices = 0;
	}
	
	public void reset() {
		resetArrays();
		resetBuffers();
	}
	
	public static void pushProjectionMatrix() {
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
	
	public static void popProjectionMatrix() {
		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
	}
	
	public static void setupPerspective(float fov) {
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		GLU.gluPerspective(fov, (float) Display.getWidth() / (float) Display.getHeight(), 0.001f, 100);
		glViewport(0, 0, Display.getWidth(), Display.getHeight());
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
	
	public static void setupOrtho() {
		setupOrtho(0, 0, Display.getWidth(), Display.getHeight());
	}
	
	public static void setupOrtho(float x1, float y1, float x2, float y2) {
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(x1, x2, y2, y1, -2, 2);
		glViewport(0, 0, Display.getWidth(), Display.getHeight());
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}
}
